# -*- coding: utf-8 -*-
"""
Created on Mon Oct 27 17:48:33 2014

@author: akusok
"""

import numpy as np


def train_cv(self, X, T, k):
    N = X.shape[0]

    idxk = []
    for i in range(k):
        idxk.append(np.arange(N)[i::k])

    datak = []
    for i in range(k):
        items = [(i+j) % k for j in range(k)]
        idx_tr = np.hstack([idxk[j] for j in items[:-2]])
        idx_vl = idxk[items[-2]]
        idx_ts = idxk[items[-1]]
        Xtr = X[idx_tr]
        Ttr = T[idx_tr]
        Xvl = X[idx_vl]
        Tvl = T[idx_vl]
        Xts = X[idx_ts]
        Tts = T[idx_ts]
        HH, HT = self._project(Xtr, Ttr)
        Hvl = self.project(Xvl)
        Hts = self.project(Xts)
        rank, nn = self._ranking(Hvl.shape[1], Hvl, Tvl)
        datak.append((HH, HT, Hvl, Tvl, Hts, Tts, rank))

    e = np.ones((nn+1,)) * -1  # errors for all numbers of neurons

    err = 0
    for HH, HT, Hvl, Tvl, _, _, _ in datak:
        Beta = self._solve_corr(HH, HT)
        Yvl = np.dot(Hvl, Beta)
        err += self._error(Yvl, Tvl) / k
    penalty = err * 0.01 / nn  # penalty is 1% of error at max(nn)
    e[nn] = err + nn * penalty

    # MYOPT function
    # [A  B  C  D  E] interval points,
    # halve the interval each time

    # initialize intervals
    A = 3
    E = nn
    l = E - A
    B = A + l/4
    C = A + l/2
    D = A + 3*l/4

    l = 1000  # run the while loop at least once
    while l > 2:
        # calculate errors at points
        for idx in [A, B, C, D, E]:
            if e[idx] == -1:  # skip already calculated errors
                err = 0
                for HH, HT, Hvl, Tvl, _, _, rank in datak:
                    rank1 = rank[:idx]
                    HH1 = HH[rank1, :][:, rank1]
                    HT1 = HT[rank1, :]
                    Beta = self._solve_corr(HH1, HT1)
                    Yvl = np.dot(Hvl[:, rank1], Beta)
                    err += self._error(Yvl, Tvl) / k
                e[idx] = err + idx * penalty

        m = min(e[A], e[B], e[C], e[D], e[E])  # find minimum element

        # halve the search interval
        if m in (e[A], e[B]):
            E = C
            C = B
        elif m in (e[D], e[E]):
            A = C
            C = D
        else:
            A = B
            E = D
        l = E - A
        B = A + l/4
        D = A + (3*l)/4

    k_opt = [n1 for n1 in [A, B, C, D, E] if e[n1] == m][0]  # find minimum index
    best_nn = rank[:k_opt]

    # get test error
    err_ts = 0
    for HH, HT, _, _, Hts, Tts, _ in datak:
        Beta = self._solve_corr(HH, HT)
        Yts = np.dot(Hts, Beta)
        err_ts += self._error(Yts, Tts) / k

    self._prune(best_nn)
    self.Beta = self._project(X, T, solve=True)[2]
#    print "%d of %d neurons selected with a Cross-Validation" % (len(best_nn), nn)
#    print "the Cross-Validation test error is %f" % err_ts
#    if len(best_nn) > nn*0.9:
#        print "Hint: try re-training with more hidden neurons"
    return err_ts





















# -*- coding: utf-8 -*-
"""
Created on Sat Oct 18 17:21:12 2014

@author: akusok
"""

from setuptools import setup
from distutils.extension import Extension
from Cython.Distutils import build_ext

import commands
import numpy
import sys



def pkgconfig(*packages, **kw):
    """Returns nicely organized stuff from PKGCONFIG.

    Found on the internet, returns a dictionary with
    libraries, library dirs, include dirs, extra arguments

    To test, run in terminal: "pkg-config --libs --cflags magma"

    To add "magma" to pkg-config:
    export PKG_CONFIG_PATH=$PKG_CONFIG_PATH:/usr/local/magma/lib/pkgconfig
    use your own path to installed magma + lib/pkgconfig
    """
    flag_map = {'-I': 'include_dirs', '-L': 'library_dirs', '-l': 'libraries'}
    for token in commands.getoutput("pkg-config --libs --cflags %s" % ' '.join(packages)).split():
        if token[:2] in flag_map:
            kw.setdefault(flag_map.get(token[:2]), []).append(token[2:])
        else:
            kw.setdefault('extra_compile_args', []).append(token)
    return kw


setup(cmdclass={'build_ext': build_ext},
      ext_modules=[
          Extension(
              "gpu_solver",
              sources=["gpu_solver.pyx",
                       "gpu_code.cpp"],
              language="c++",
              extra_compile_args=pkgconfig("magma")["extra_compile_args"],
              include_dirs=[numpy.get_include()] + pkgconfig("magma")["include_dirs"],
              libraries=pkgconfig("magma")["libraries"],
              library_dirs=pkgconfig("magma")["library_dirs"])
          ]
      )



# -*- coding: utf-8 -*-
"""
Created on Thu Apr  2 21:12:46 2015

@author: akusok
"""

import numpy as np
import csv
from tables import open_file, Atom, Filters


def normalize_hdf5(h5file, mean=None, std=None, batch=None):
    """Calculates and applies normalization to data in HDF5 file.

    :param mean: - known vector of mean values
    :param std: - known vector of standard deviations
    :param batch: - number of rows to read at once, default is a native batch size
    """

    h5 = open_file(h5file, "a")
    for node in h5.walk_nodes():
        pass  # find a node with whatever name
    dt = node.dtype
    N, d = node.shape  # HDF5 files are transposed, for Matlab compatibility
    if batch is None:
        batch = node.chunkshape[0]
    nb = N/batch
    if N > nb*batch:
        nb += 1  # add last incomplete step

    if mean is None or std is None:
        if node.attrs.mean is None:  # data was not normalized before
            print "calculating mean and standard deviation of data"
            E_x = np.zeros((d,), dtype=np.float64)
            E_x2 = np.zeros((d,), dtype=np.float64)
            for b in xrange(nb):
                start = b*batch
                step = min(batch, N-start)
                X1 = node[start: start+step, :].astype(np.float64)
                E_x += np.mean(X1, 0) * (1.0*step/N)
                E_x2 += np.mean(X1**2, 0) * (1.0*step/N)
            mean = E_x
            E2_x = E_x**2
            std = (E_x2 - E2_x)**0.5
            node.attrs.mean = mean
            node.attrs.std = std
            return mean, std
        else:  # data is already normalized
            print "data was already normalized, returning 'mean', 'std' parameters"
            print "if you want to run normalization anyway, call the function with 'mean' and 'std' params"
            return node.attrs.mean, node.attrs.std
    else:
        assert len(mean) == d, "Incorrect lenght of a vector of means: %d expected, %d found" % (d, len(mean))
        assert len(std) == d, "Incorrect lenght of a vector of standard deviations: %d expected, %d found" % (d, len(std))
        node.attrs.mean = mean
        node.attrs.std = std
    std[std == 0] = 1  # prevent division by zero for std=0

    print "applying normalization"
    for b in xrange(nb):
        start = b*batch
        step = min(batch, N-start)
        X = node[start: start+step].astype(np.float64)
        X = (X - mean) / std
        node[start: start+step] = X.astype(dt)

    h5.close()  # closing file
    return mean, std


#def oversample(data, targets, classes):
#    pass


def make_hdf5(data, h5file, dtype=np.float64, delimiter=" ", skiprows=0, comp_level=0):
    """Makes an HDF5 file from whatever given data.

    :param data: - input data in Numpy.ndarray or filename, or a shape tuple
    :param h5file: - name (and path) of the output HDF5 file
    :param delimiter: - data delimiter for text, csv files
    :param comp_level: - compression level of the HDF5 file
    """
    assert comp_level < 10, "Compression level must be 0-9 (0 for no compression)"
    fill = ""

    # open data file
    if isinstance(data, np.ndarray):
        X = data
    elif isinstance(data, basestring) and data[-3:] in ['npy']:
        X = np.load(data)
    elif isinstance(data, basestring) and data[-3:] in ['.gz', 'bz2']:
        X = np.loadtxt(data, dtype=dtype, delimiter=delimiter, skiprows=skiprows)
    elif isinstance(data, basestring) and data[-3:] in ['txt', 'csv']:
        # iterative out-of-memory loader for huge .csv/.txt files
        fill = "iter"
        # check data dimensionality
        with open(data, "rU") as f:
            for _ in xrange(skiprows):
                f.readline()
            reader = csv.reader(f, delimiter=delimiter)
            for line in reader:
                X = np.fromiter(line, dtype=dtype)
                break
    elif isinstance(data, tuple) and len(data) == 2:
        X = np.empty((1, 1))
        fill = "empty"
    else:
        assert False, "Input data must be Numpy ndarray, .npy file, or .txt/.csv text file (compressed .gz/.bz2)"

    # process data
    if len(X.shape) == 1:
        X = X[:, np.newaxis]
    assert len(X.shape) == 2, "Data in Numpy ndarray must have 2 dimensions"
    # create hdf5 file
    if comp_level > 0:
        flt = Filters(complevel=comp_level, shuffle=True)
    else:
        flt = Filters(complevel=0)
    h5 = open_file(h5file, "w")
    a = Atom.from_dtype(np.dtype(dtype), dflt=0)
    # write data to hdf5 file
    if fill == "iter":  # iteratively fill the data
        h5data = h5.create_earray(h5.root, "data", a, (0, X.shape[0]), filters=flt)
        with open(data, "rU") as f:
            for _ in xrange(skiprows):
                f.readline()
            reader = csv.reader(f, delimiter=delimiter)
            for line in reader:
                row = np.fromiter(line, dtype=dtype)
                h5data.append(row[np.newaxis, :])
    elif fill == "empty":  # no fill at all
        h5data = h5.create_carray(h5.root, "data", a, data, filters=flt)
    else:  # write whole data matrix
        h5data = h5.create_carray(h5.root, "data", a, X.shape, filters=flt)
        h5data[:] = X
    # close the file
    h5data.attrs.mean = None
    h5data.attrs.std = None
    h5.flush()
    h5.close()


if __name__ == "__main__":
    # def make_hdf5(data, h5file, dtype=np.float64, delimiter=" ", skiprows=0, comp_level=0):
    # make_hdf5("textfile.txt", "text.h5")
    # make_hdf5("textfile.txt", "textz.h5", comp_level=3)
    normalize_hdf5("text.h5")
    print "Done!"



# -*- coding: utf-8 -*-
"""
Created on Mon Oct 27 14:12:41 2014

@author: akusok
"""

from unittest import TestCase
import numpy as np

from hpelm import ELM


class TestCorrectness(TestCase):

    def test_1_NonNumpyInputs_RaiseError(self):
        X = np.array([['1', '2'], ['3', '4'], ['5', '6']])
        T = np.array([[1], [2], [3]])
        elm = ELM(2, 1)
        elm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_2_NonNumpyTargets_RaiseError(self):
        X = np.array([[1, 2], [3, 4], [5, 6]])
        T = np.array([['a'], ['b'], ['c']])
        elm = ELM(2, 1)
        elm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_3_OneDimensionInputs_RunsCorrectly(self):
        X = np.array([1, 2, 3])
        T = np.array([[1], [2], [3]])
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        elm.train(X, T)

    def test_4_OneDimensionTargets_RunsCorrectly(self):
        X = np.array([1, 2, 3])
        T = np.array([1, 2, 3])
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        elm.train(X, T)

    def test_5_WrongDimensionalityInputs_RaiseError(self):
        X = np.array([[1, 2], [3, 4], [5, 6]])
        T = np.array([[1], [2], [3]])
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_6_WrongDimensionalityTargets_RaiseError(self):
        X = np.array([[1, 2], [3, 4], [5, 6]])
        T = np.array([[1], [2], [3]])
        elm = ELM(1, 2)
        elm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_7_ZeroInputs_RunsCorrectly(self):
        X = np.array([[0, 0], [0, 0], [0, 0]])
        T = np.array([1, 2, 3])
        elm = ELM(2, 1)
        elm.add_neurons(1, "lin")
        elm.train(X, T)

    def test_8_OneDimensionTargets_RunsCorrectly(self):
        X = np.array([[1, 2], [3, 4], [5, 6]])
        T = np.array([[0], [0], [0]])
        elm = ELM(2, 1)
        elm.add_neurons(1, "lin")
        elm.train(X, T)

    def test_9_TrainWithoutNeurons_RaiseError(self):
        X = np.array([1, 2, 3])
        T = np.array([1, 2, 3])
        elm = ELM(1, 1)
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_10_DifferentNumberOfSamples_RaiseError(self):
        X = np.array([[1, 2], [3, 4], [5, 6]])
        T = np.array([[1], [2]])
        elm = ELM(2, 1)
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_11_LinearNeurons_MoreThanInputs_Truncated(self):
        elm = ELM(2, 1)
        elm.add_neurons(3, "lin")
        self.assertEqual(2, elm.neurons[0][1])

    def test_12_LinearNeurons_DefaultMatrix_Identity(self):
        elm = ELM(4, 1)
        elm.add_neurons(3, "lin")
        np.testing.assert_array_almost_equal(np.eye(4, 3), elm.neurons[0][2])

    def test_13_SLFN_AddLinearNeurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        self.assertEquals("lin", elm.neurons[0][0])

    def test_14_SLFN_AddSigmoidalNeurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "sigm")
        self.assertEquals("sigm", elm.neurons[0][0])

    def test_15_SLFN_AddTanhNeurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "tanh")
        self.assertEquals("tanh", elm.neurons[0][0])

    def test_16_SLFN_AddRbfL1Neurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "rbf_l1")
        self.assertEquals("rbf_l1", elm.neurons[0][0])

    def test_17_SLFN_AddRbfL2Neurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "rbf_l2")
        self.assertEquals("rbf_l2", elm.neurons[0][0])

    def test_18_SLFN_AddRbfLinfNeurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "rbf_linf")
        self.assertEquals("rbf_linf", elm.neurons[0][0])

    def test_19_SLFN_AddUfuncNeurons_GotThem(self):
        elm = ELM(1, 1)
        func = np.frompyfunc(lambda a: a+1, 1, 1)
        elm.add_neurons(1, func)
        self.assertIs(func, elm.neurons[0][0])

    def test_20_SLFN_AddTwoNeuronTypes_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        elm.add_neurons(1, "sigm")
        self.assertEquals(2, len(elm.neurons))
        ntypes = [nr[0] for nr in elm.neurons]
        self.assertIn("lin", ntypes)
        self.assertIn("sigm", ntypes)

    def test_21_SLFN_AddNeuronsTwice_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        elm.add_neurons(1, "lin")
        self.assertEquals(1, len(elm.neurons))
        self.assertEquals(2, elm.neurons[0][1])

    def test_22_AddNeurons_InitBias_BiasInModel(self):
        elm = ELM(1, 1)
        bias = np.array([1, 2, 3])
        elm.add_neurons(3, "sigm", None, bias)
        np.testing.assert_array_almost_equal(bias, elm.neurons[0][3])

    def test_23_AddNeurons_InitW_WInModel(self):
        elm = ELM(2, 1)
        W = np.array([[1, 2, 3], [4, 5, 6]])
        elm.add_neurons(3, "sigm", W, None)
        np.testing.assert_array_almost_equal(W, elm.neurons[0][2])

    def test_24_AddNeurons_InitDefault_BiasWNotZero(self):
        elm = ELM(2, 1)
        elm.add_neurons(3, "sigm")
        W = elm.neurons[0][2]
        bias = elm.neurons[0][3]
        self.assertGreater(np.sum(np.abs(W)), 0.001)
        self.assertGreater(np.sum(np.abs(bias)), 0.001)

    def test_25_AddNeurons_InitTwiceBiasW_CorrectlyMerged(self):
        elm = ELM(2, 1)
        W1 = np.random.rand(2, 3)
        W2 = np.random.rand(2, 4)
        bias1 = np.random.rand(3,)
        bias2 = np.random.rand(4,)
        elm.add_neurons(3, "sigm", W1, bias1)
        elm.add_neurons(4, "sigm", W2, bias2)
        np.testing.assert_array_almost_equal(np.hstack((W1, W2)), elm.neurons[0][2])
        np.testing.assert_array_almost_equal(np.hstack((bias1, bias2)), elm.neurons[0][3])




























#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Sun Oct 19 16:29:09 2014

@author: akusok
"""

import numpy as np
import os
from hpelm import ELM

curdir = os.path.dirname(__file__)
pX = os.path.join(curdir, "../datasets/Unittest-Iris/iris_data.txt")
pY = os.path.join(curdir, "../datasets/Unittest-Iris/iris_classes.txt")

X = np.loadtxt(pX)
Y = np.loadtxt(pY)

elm = ELM(4,3)
elm.add_neurons(15, "sigm")
elm.train(X, Y, "c")
Yh = elm.predict(X)
acc = float(np.sum(Y.argmax(1) == Yh.argmax(1))) / Y.shape[0]
print "Iris dataset training error: %.1f%%" % (100-acc*100)


 # -*- coding: utf-8 -*-
"""
Created on Mon Oct 27 17:48:33 2014

@author: akusok
"""

import numpy as np
from slfn import SLFN
from hpelm.modules import mrsr, mrsr2
from mss_v import train_v
from mss_cv import train_cv
from mss_loo import train_loo


class ELM(SLFN):
    """Interface for training Extreme Learning Machines.
    """

    def train(self, X, T, *args, **kwargs):
        """Universal training interface for ELM model with model structure selection.

        :param X: input data matrix
        :param T: target data matrix

        Model structure selection (exclusive, choose one)
        :param "V": use validation set
        :param "CV": use cross-validation
        :param "LOO": use leave-one-out validation

        Additional parameters for model structure selecation
        :param Xv: validation data X ("V")
        :param Tv: validation targets T ("V")
        :param k: number of splits ("CV")

        Ranking of hidden neurons
        :param "OP": use Optimal Pruning (OP-ELM)
        :param "kmax": maximum number of neurons (with "OP")

        System setup
        :param "classification"/"c": build ELM for classification
        :param "weighted classification"/"wc": build ELM with weights assigned to classes
        :param w: weights of classes for "wc"
        :param "multiclass"/"mc": build ELM for multiclass classification
        :param "adaptive"/"ad": build adaptive ELM for non-stationary model
        :param "batch": batch size for adaptive ELM (sliding window step size)
        """

        assert len(self.neurons) > 0, "Add neurons to ELM before training it"
        X, T = self._checkdata(X, T)
        args = [a.upper() for a in args]  # make all arguments upper case

        # kind of "enumerators", try to use only inside that script
        MODELSELECTION = None  # V / CV / MCCV / LOO / None
        ADAPTIVE = False  # batch / None

        # reset parameters
        self.ranking = None
        self.kmax_op = None
        self.classification = None  # c / wc / mc
        self.weights_wc = None  # weigths for weighted classification

        # check exclusive parameters
        assert len(set(args).intersection(set(["V", "CV", "LOO"]))) <= 1, "Use only one of V / CV / LOO"
        assert len(set(args).intersection(set(["C", "WC", "MC"]))) <= 1, "Use only one of \
            C (classification) / MC (multiclass) / WC (weighted classification)"

        # parse parameters
        for a in args:
            if a == "V":  # validation set
                assert "Xv" in kwargs.keys(), "Provide validation dataset (Xv)"
                assert "Tv" in kwargs.keys(), "Provide validation targets (Tv)"
                Xv = kwargs['Xv']
                Tv = kwargs['Tv']
                Xv, Tv = self._checkdata(Xv, Tv)
                MODELSELECTION = "V"
            if a == "CV":
                assert "k" in kwargs.keys(), "Provide Cross-Validation number of splits (k)"
                k = kwargs['k']
                assert k >= 3, "Use at least k=3 splits for Cross-Validation"
                MODELSELECTION = "CV"
            if a == "LOO":
                MODELSELECTION = "LOO"
            if a == "OP":
                self.ranking = "OP"
                if "kmax" in kwargs.keys():
                    self.kmax_op = int(kwargs["kmax"])
            if a == "C":
                assert self.targets > 1, "Classification targets must have 1 output per class"
                self.classification = "c"
            if a == "WC":
                assert self.targets > 1, "Classification targets must have 1 output per class"
                assert "w" in kwargs.keys(), "Provide class weights for weighted classification"
                w = kwargs['w']
                assert len(w) == T.shape[1], "Number of class weights differs from number of target classes"
                self.weights_wc = w
                self.classification = "wc"
            if a == "MC":
                assert self.targets > 1, "Classification targets must have 1 output per class"
                self.classification = "mc"
            # if a in ("A", "AD", "ADAPTIVE"):
            #     assert "batch" in kwargs.keys(), "Provide batch size for adaptive ELM model (batch)"
            #     batch = kwargs['batch']
            #     ADAPTIVE = True

        # use "train_x" method which borrows _project(), _error() from the "self" object
        if MODELSELECTION == "V":
            train_v(self, X, T, Xv, Tv)
        elif MODELSELECTION == "CV":
            train_cv(self, X, T, k)
        elif MODELSELECTION == "LOO":
            train_loo(self, X, T)
        else:
            self._train(X, T)

    def _train(self, X, T):
        """Most basic training algorithm for an ELM.
        """
        self.Beta = self._project(X, T, solve=True)[2]

    def _project(self, X, T, solve=False):
        """Create HH, HT matrices and computes solution Beta.

        An ELM-specific projection for all usage cases.
        Returns solution Beta if solve=True.
        Runs on GPU if self.accelerator="GPU".
        Performs balanced classification if self.classification="cb".
        """
        # initialize
        nn = sum([n1[1] for n1 in self.neurons])
        batch = max(self.batch, nn)
        if X.shape[0] % batch > 0:
            nb = X.shape[0]/batch + 1
        else:
            nb = X.shape[0]/batch

        # GPU script
        def proj_gpu(self, X, T, getBeta, nn, nb):
            s = self.magma_solver.GPUSolver(nn, self.targets, self.alpha)
            for X0, T0 in zip(np.array_split(X, nb, axis=0),
                              np.array_split(T, nb, axis=0)):
                H0 = self.project(X0)
                s.add_data(H0, T0)
            HH, HT = s.get_corr()
            if getBeta:
                Beta = s.solve()
            else:
                Beta = None
            return HH, HT, Beta

        # CPU script
        def proj_cpu(self, X, T, getBeta, nn, nb):
            HH = np.zeros((nn, nn))
            HT = np.zeros((nn, self.targets))
            HH.ravel()[::nn+1] += self.alpha  # add to matrix diagonal trick
            for X0, T0 in zip(np.array_split(X, nb, axis=0),
                              np.array_split(T, nb, axis=0)):
                H0 = self.project(X0)
                HH += np.dot(H0.T, H0)
                HT += np.dot(H0.T, T0)
            if getBeta:
                Beta = self._solve_corr(HH, HT)
            else:
                Beta = None
            return HH, HT, Beta

        # run scripts
        if self.classification == "cb":  # balanced classification wrapper
            ns = T.sum(axis=0).astype(np.float64)  # number of samples in classes
            wc = (ns / ns.sum())**-1  # weights of classes
            HH = np.zeros((nn, nn))  # init data holders
            HT = np.zeros((nn, self.targets))
            for i in range(wc.shape[0]):  # iterate over each particular class
                idxc = T[:, i] == 1
                Xc = X[idxc]
                Tc = T[idxc]
                if self.accelerator == "GPU":
                    HHc, HTc, _ = proj_gpu(self, Xc, Tc, False, nn, nb)
                else:
                    HHc, HTc, _ = proj_cpu(self, Xc, Tc, False, nn, nb)
                HH += HHc * wc[i]
                HT += HTc * wc[i]
            if solve:  # obtain solution
                Beta = self._solve_corr(HH, HT)
        else:
            if self.accelerator == "GPU":
                HH, HT, Beta = proj_gpu(self, X, T, solve, nn, nb)
            else:
                HH, HT, Beta = proj_cpu(self, X, T, solve, nn, nb)

        # return results
        if solve:
            return HH, HT, Beta
        else:
            return HH, HT

    def _error(self, Y, T, R=None):
        """Returns regression/classification/multiclass error, also for PRESS.

        An ELM-specific error with PRESS support.
        """
        if R is None:  # normal classification error
            if self.classification == "c":
                err = np.mean(Y.argmax(1) != T.argmax(1))
            elif self.classification == "wc":  # weighted classification
                c = T.shape[1]
                errc = np.zeros(c)
                for i in xrange(c):  # per-class MSE
                    idx = np.where(T[:, i] == 1)[0]
                    if len(idx) > 0:
                        errc[i] = np.mean(Y[idx].argmax(1) != i)
                err = np.mean(errc * self.weights_wc)
            elif self.classification == "mc":
                err = np.mean((Y > 0.5) != (T > 0.5))
            else:
                err = np.mean((Y - T)**2)
        else:  # LOO_PRESS error
            if self.classification == "c":
                err = (Y.argmax(1) != T.argmax(1)).astype(np.float) / R.ravel()
                err = np.mean(err**2)
            elif self.classification == "wc":  # balanced classification
                c = T.shape[1]
                errc = np.zeros(c)
                for i in xrange(c):  # per-class MSE
                    idx = np.where(T[:, i] == 1)[0]
                    if len(idx) > 0:
                        t = (Y[idx].argmax(1) != i).astype(np.float) / R[idx].ravel()
                        errc[i] = np.mean(t**2)
                err = np.mean(errc * self.weights_wc)
            elif self.classification == "mc":
                err = ((Y > 0.5) != (T > 0.5)).astype(np.float) / R.reshape((-1, 1))
                err = np.mean(err**2)
            else:
                err = (Y - T) / R.reshape((-1, 1))
                err = np.mean(err**2)
        assert not np.isnan(err), "Error is NaN at %s" % self.classification
        return err

    def _ranking(self, nn, H=None, T=None):
        """Return ranking of hidden neurons; random or OP.
        """
        if self.ranking == "OP":
            if self.kmax_op is None:  # set maximum number of neurons
                self.kmax_op = nn
            else:  # or set a limited number of neurons
                nn = self.kmax_op
            if T.shape[1] > 10:  # fast mrsr for less outputs but O(2^t) in outputs
                rank = mrsr(H, T, self.kmax_op)
            else:  # slow mrsr for many outputs but O(t) in outputs
                rank = mrsr2(H, T, self.kmax_op)
        else:
            rank, nn = super(ELM, self)._ranking(nn)
        return rank, nn





























# -*- coding: utf-8 -*-
"""
Created on Mon Oct 27 17:48:33 2014

@author: akusok
"""

import numpy as np
from time import time
from hpelm.modules import make_hdf5
from tables import open_file
from slfn import SLFN


class HPELM(SLFN):
    """Interface for training Extreme Learning Machines.
    """

    def train(self, X, T, *args, **kwargs):
        """Universal training interface for ELM model with model structure selection.

        :param X: input data matrix
        :param T: target data matrix

        Model structure selection (exclusive, choose one)
        :param "V": use validation set
        :param "CV": use cross-validation

        Additional parameters for model structure selecation
        :param Xv: validation data X ("V")
        :param Tv: validation targets T ("V")
        :param k: number of splits ("CV")

        System setup
        :param "c": build ELM for classification
        :param "cb": build ELM with balanced classification
        :param "mc": build ELM for multiclass classification
        :param "adaptive"/"ad": build adaptive ELM for non-stationary model
        :param "batch": batch size for adaptive ELM (sliding window step size)
        """

        assert len(self.neurons) > 0, "Add neurons to ELM before training it"
        X, T = self._checkdata(X, T)
        args = [a.upper() for a in args]  # make all arguments upper case

        # kind of "enumerators", try to use only inside that script
        MODELSELECTION = None  # V / CV / None
        ADAPTIVE = False  # batch / None

        # reset parameters
        self.ranking = None
        self.kmax_op = None
        self.classification = None  # c / wc / mc
        self.weights_wc = None  # weigths for weighted classification

        # check exclusive parameters
        assert len(set(args).intersection(set(["C", "WC", "MC"]))) <= 1, "Use only one of \
            C (classification) / MC (multiclass) / WC (weighted classification)"

        # parse parameters
        for a in args:
            if a == "C":
                assert self.targets > 1, "Classification targets must have 1 output per class"
                self.classification = "c"
            if a == "WC":
                assert self.targets > 1, "Classification targets must have 1 output per class"
                assert "w" in kwargs.keys(), "Provide class weights for weighted classification"
                w = kwargs['w']
                assert len(w) == T.shape[1], "Number of class weights differs from number of target classes"
                self.weights_wc = w
                self.classification = "wc"
            if a == "MC":
                self.classification = "mc"
            # if a in ("A", "AD", "ADAPTIVE"):
            #     assert "batch" in kwargs.keys(), "Provide batch size for adaptive ELM model (batch)"
            #     batch = kwargs['batch']
            #     ADAPTIVE = True

        # use "train_x" method which borrows _project(), _error() from the "self" object
        self._train(X, T)

    def _train(self, X, T):
        """Most basic training algorithm for an ELM.
        """
        self.Beta = self._project(X, T, solve=True)[2]

    def _makeh5(self, h5name, N, d):
        """Creates HDF5 file opened in append mode.
        """
        make_hdf5((N, d), h5name)
        h5 = open_file(h5name, "a")
        self.opened_hdf5.append(h5)
        for node in h5.walk_nodes():
            pass  # find a node with whatever name
        return node


    def predict(self, X1, Y1):
        """Iterative predict which saves data to HDF5.
        """
        assert self.Beta is not None, "Train ELM before predicting"
        X, _ = self._checkdata(X1, None)
        N = X.shape[0]
        Y = self._makeh5(Y1, N, self.targets)
        nn = np.sum([n1[1] for n1 in self.neurons])
        batch = max(self.batch, nn)
        nb = N / batch  # number of batches
        if batch > N * nb:
            nb += 1
        t = time()
        t0 = time()
        eta = 0
        for b in xrange(nb):
            eta = int(((time()-t0) / (b+0.0000001)) * (nb-b))
            if time() - t > self.tprint:
                print "processing batch %d/%d, eta %d:%02d:%02d" % (b+1, nb, eta/3600, (eta % 3600)/60, eta % 60)
                t = time()
            start = b*batch
            stop = min((b+1)*batch, N)
            Xb = X[start:stop].astype(np.float64)
            Hb = self.project(Xb)
            Y[start:stop] = Hb.dot(self.Beta)
        # close HDF5 for Y1
        h5 = self.opened_hdf5.pop()
        h5.close()


    def _project(self, X, T, solve=False, wwc=None):
        """Create HH, HT matrices and computes solution Beta.

        HPELM-specific parallel projection.
        Returns solution Beta if solve=True.
        Runs on GPU if self.accelerator="GPU".
        Performs balanced classification if self.classification="cb".
        """
        # initialize
        nn = np.sum([n1[1] for n1 in self.neurons])
        batch = max(self.batch, nn)
        N = X.shape[0]
        nb = N / batch  # number of batches
        if batch > N * nb:
            nb += 1
        HH = np.zeros((nn, nn))  # init data holders
        HT = np.zeros((nn, self.targets))
        if self.classification == "wc":  # weighted classification initialization
            ns = np.zeros((self.targets,))
            for b in xrange(nb):  # batch sum is much faster
                ns += T[b*batch: (b+1)*batch].sum(axis=0)
            wc = (float(ns.sum()) / ns) * self.weights_wc  # class weights normalized to number of samples
            wc = wc**0.5  # because it gets applied twice

        if wwc is not None:
            wc = wwc

        if self.accelerator == "GPU":
            s = self.magma_solver.GPUSolver(nn, self.targets, self.alpha)
        else:
            HH.ravel()[::nn+1] += self.alpha  # add to matrix diagonal trick

        # main loop over all the data
        t = time()
        t0 = time()
        eta = 0
        for b in xrange(nb):
            eta = int(((time()-t0) / (b+0.0000001)) * (nb-b))
            if time() - t > self.tprint:
                print "processing batch %d/%d, eta %d:%02d:%02d" % (b+1, nb, eta/3600, (eta % 3600)/60, eta % 60)
                t = time()
            start = b*batch
            stop = min((b+1)*batch, N)
            Xb = X[start:stop].astype(np.float64)
            Tb = T[start:stop].astype(np.float64)
            Hb = self.project(Xb)

            if self.classification == "wc":  # apply per-sample weighting
                ci = Tb.argmax(1)
                Hb *= wc[ci, None]
                Tb *= wc[ci, None]
            if self.accelerator == "GPU":
                s.add_data(Hb, Tb)
            else:
                HH += np.dot(Hb.T, Hb)
                HT += np.dot(Hb.T, Tb)

        # get computed matrices back
        if self.accelerator == "GPU":
            HH, HT = s.get_corr()
            if solve:
                Beta = s.solve()
            s.finalize()
        elif solve:
            Beta = self._solve_corr(HH, HT)

        # return results
        if solve:
            return HH, HT, Beta
        else:
            return HH, HT


    def _error(self, Y1, T1, H1=None, Beta=None, rank=None):
        """Do projection and calculate error in batch mode.

        HPELM-specific iterative error for all usage cases.
        Can be _error(Y, T) or _error(None, T, H, Beta, rank)

        :param T: - true targets for error calculation
        :param H: - projected data for error calculation
        :param Beta: - current projection matrix
        :param rank: - selected neurons (= columns of H)
        """
        if Y1 is None:
            H, T = self._checkdata(H1, T1)
            assert rank.shape[0] == Beta.shape[0], "Wrong dimension of Beta for the given ranking"
            assert T.shape[1] == Beta.shape[1], "Wrong dimension of Beta for the given targets"
            nn = rank.shape[0]
        else:
            _, Y = self._checkdata(None, Y1)
            _, T = self._checkdata(None, T1)
            nn = np.sum([n1[1] for n1 in self.neurons])
        N = T.shape[0]
        batch = max(self.batch, nn)
        nb = N / batch  # number of batches
        if batch > N * nb:
            nb += 1

        if self.classification == "c":
            err = 0
            for b in xrange(nb):
                start = b*batch
                stop = min((b+1)*batch, N)
                Tb = np.array(T[start:stop])
                if Y1 is None:
                    Hb = H[start:stop, rank]
                    Yb = np.dot(Hb, Beta)
                else:
                    Yb = np.array(Y[start:stop])
                errb = np.mean(Yb.argmax(1) != Tb.argmax(1))
                err += errb * float(stop-start)/N

        elif self.classification == "wc":  # weighted classification
            c = T.shape[1]
            errc = np.zeros(c)
            for b in xrange(nb):
                start = b*batch
                stop = min((b+1)*batch, N)
                Tb = np.array(T[start:stop])
                if Y1 is None:
                    Hb = H[start:stop, rank]
                    Yb = np.dot(Hb, Beta)
                else:
                    Yb = np.array(Y[start:stop])
                for i in xrange(c):  # per-class MSE
                    idxc = Tb[:, i] == 1
                    errb = np.mean(Yb[idxc].argmax(1) != i)
                    errc[i] += errb * float(stop-start)/N
            err = np.mean(errc * self.weights_wc)

        elif self.classification == "mc":
            err = 0
            for b in xrange(nb):
                start = b*batch
                stop = min((b+1)*batch, N)
                Tb = np.array(T[start:stop])
                if Y1 is None:
                    Hb = H[start:stop, rank]
                    Yb = np.dot(Hb, Beta)
                else:
                    Yb = np.array(Y[start:stop])
                errb = np.mean((Yb > 0.5) != (Tb > 0.5))
                err += errb * float(stop-start)/N

        else:  # MSE error
            err = 0
            for b in xrange(nb):
                start = b*batch
                stop = min((b+1)*batch, N)
                Tb = T[start:stop]
                if Y1 is None:
                    Hb = H[start:stop, rank]
                    Yb = np.dot(Hb, Beta)
                else:
                    Yb = Y[start:stop]
                errb = np.mean((Tb - Yb)**2)
                err += errb * float(stop-start)/N

        return err


    def train_hpv(self, HH, HT, Xv, Tv, steps=10):
        X, T = self._checkdata(Xv, Tv)
        N = X.shape[0]
        nn = HH.shape[0]

        nns = np.logspace(np.log(3), np.log(nn), steps, base=np.e, endpoint=True)
        nns = np.ceil(nns).astype(np.int)
        nns = np.unique(nns)  # numbers of neurons to check
        print nns
        k = nns.shape[0]
        err = np.zeros((k,))  # errors for these numbers of neurons

        batch = max(self.batch, nn)
        nb = N / batch  # number of batches
        if batch > N * nb:
            nb += 1

        Betas = []  # keep all betas in memory
        for l in nns:
            Betas.append(self._solve_corr(HH[:l, :l], HT[:l, :]))

        t = time()
        t0 = time()
        eta = 0
        for b in xrange(nb):
            eta = int(((time()-t0) / (b+0.0000001)) * (nb-b))
            if time() - t > self.tprint:
                print "processing batch %d/%d, eta %d:%02d:%02d" % (b+1, nb, eta/3600, (eta % 3600)/60, eta % 60)
                t = time()
            start = b*batch
            stop = min((b+1)*batch, N)
            alpha = float(stop-start)/N
            Tb = np.array(T[start:stop])
            Xb = np.array(X[start:stop])
            Hb = self.project(Xb)
            for i in xrange(k):
                hb1 = Hb[:, :nns[i]]
                Yb = np.dot(hb1, Betas[i])
                err[i] += self._error(Yb, Tb) * alpha

        k_opt = np.argmin(err)
        best_nn = nns[k_opt]
        self._prune(np.arange(best_nn))
        self.Beta = Betas[k_opt]
        del Betas
        print "%d of %d neurons selected with a validation set" % (best_nn, nn)
        if best_nn > nn*0.9:
            print "Hint: try re-training with more hidden neurons"
        return nns, err


    def train_myhpv(self, HH, HT, Xv, Tv, steps=10):
        X, T = self._checkdata(Xv, Tv)
        N = X.shape[0]
        nn = HH.shape[0]

        nns = np.logspace(np.log(3), np.log(nn), steps, base=np.e, endpoint=True)
        nns = np.ceil(nns).astype(np.int)
        nns = np.unique(nns)  # numbers of neurons to check
        k = nns.shape[0]
        err = np.zeros((k, 2, 2))  # errors for these numbers of neurons

        batch = max(self.batch, nn)
        nb = N / batch  # number of batches
        if batch > N * nb:
            nb += 1

        Betas = []  # keep all betas in memory
        for l in nns:
            Betas.append(self._solve_corr(HH[:l, :l], HT[:l, :]))

        t = time()
        t0 = time()
        eta = 0
        for b in xrange(nb):
            eta = int(((time()-t0) / (b+0.0000001)) * (nb-b))
            if time() - t > self.tprint:
                print "processing batch %d/%d, eta %d:%02d:%02d" % (b+1, nb, eta/3600, (eta % 3600)/60, eta % 60)
                t = time()
            start = b*batch
            stop = min((b+1)*batch, N)
            Tb = np.array(T[start:stop])
            Xb = np.array(X[start:stop])
            Hb = self.project(Xb)
            Tc = np.argmax(Tb, axis=1)
            for i in xrange(k):
                hb1 = Hb[:, :nns[i]]
                Yb = np.dot(hb1, Betas[i])
                Yc = np.argmax(Yb, axis=1)
                err[i, 0, 0] += np.sum((Tc == 0)*(Yc == 0))
                err[i, 0, 1] += np.sum((Tc == 0)*(Yc == 1))
                err[i, 1, 0] += np.sum((Tc == 1)*(Yc == 0))
                err[i, 1, 1] += np.sum((Tc == 1)*(Yc == 1))

        return nns, err, N
























"""
Created on Thu Mar 19 16:55:19 2015

@author: akusok
"""

import cython

import numpy as np
cimport numpy as np

cdef extern from "gpu_code.h":
    cdef void solve_corr (int n, int nrhs, double* A, double* B, double* X)
    cdef cppclass GpuSolver:
        GpuSolver( int, int, double* A, double* B) except +
        void add_data( int m, double* X, double* T )
        void get_corr( double* XX, double* XT )
        void solve( double* X )
        void finalize()


cdef class GPUSolver:

    cdef GpuSolver *solverptr
    cdef int n, nrhs
    
    def __init__(self, nn, outs, norm=1E-9):
        self.n = nn
        self.nrhs = outs
        cdef np.ndarray[double, ndim=2, mode="c"] A = np.zeros((self.n, self.n))
        cdef np.ndarray[double, ndim=2, mode="c"] B = np.zeros((self.nrhs, self.n))
        A.ravel()[::self.n+1] += norm
        self.solverptr = new GpuSolver( self.n, self.nrhs, &A[0,0], &B[0,0] )

    def finalize(self):
        self.solverptr.finalize()
        
    def add_data(self,
                 np.ndarray[double, ndim=2, mode="c"] X not None,
                 np.ndarray[double, ndim=2, mode="c"] T not None):
        cdef int m = X.shape[0]
        # seems to require Fortran ordering but in "c" mode
        T = T.T.reshape(m, self.nrhs).copy()
        X = X.T.reshape(m, self.n).copy()
        self.solverptr.add_data( m, &X[0,0], &T[0,0] )
        
    def get_corr(self):
        cdef np.ndarray[double, ndim=2, mode="c"] XX = np.empty((self.n, self.n))
        cdef np.ndarray[double, ndim=2, mode="c"] XT = np.empty((self.nrhs, self.n))
        self.solverptr.get_corr( &XX[0,0], &XT[0,0] )
        XX = np.ascontiguousarray(XX.T)
        XT = np.ascontiguousarray(XT.T)
        return XX, XT
        
    def solve(self):
        cdef np.ndarray[double, ndim=2, mode="c"] B = np.empty((self.nrhs, self.n))
        self.solverptr.solve( &B[0,0] );
        return B.T  # transpose back

        
def gpu_solve(np.ndarray[double, ndim=2, mode="c"] XX not None,
              np.ndarray[double, ndim=2, mode="c"] XT not None,
              double norm):              
    cdef int n = XT.shape[0]
    cdef int nrhs = XT.shape[1]
    cdef np.ndarray[double, ndim=2, mode="c"] B = np.empty((nrhs, n))
    
    XX.ravel()[::n+1] += norm
    XT = XT.T.reshape(n,nrhs)  # transpose and cast to 'c' ordering
    solve_corr(n, nrhs, &XX[0,0], &XT[0,0], &B[0,0])
    B = B.reshape(nrhs,n).T
    return B  # transpose














# -*- coding: utf-8 -*-
"""
Better MRSR implementation according to:
'Common subset selection of inputs in multiresponse regression" by
Timo Similä and Jarkko Tikka, 
International Joint Conference on Neural Networks 2006

Linear complexity with the number of targets.

Created on Sun Jan 26 13:48:54 2014
@author: Anton Akusok
"""

import numpy as np
from scipy import optimize
from scipy.linalg import lu_factor, lu_solve


def mrsr2(X, T, kmax, norm=2):

    # initializing
    ins = X.shape[1]
    outs = T.shape[1]
    X = np.array(X, order='F')  # Fortran ordering is good for operating columns 
    XA = np.empty(X.shape, order='F')
    XX = np.dot(X.T, X)        
    XT = np.dot(X.T, T)
    rank = []  # active inputs list
    nonrank = range(ins)
    W = np.zeros((ins, outs))  # current projection estimator
    Y = np.zeros(T.shape)  # current target estimator
    Yk1 = np.zeros(T.shape)
    Wk1 = np.zeros((1, outs))
    j_current = None  # currently added input dimension
    kmax = min(kmax, X.shape[0])    
    
    # get ranking
    for _ in xrange(kmax):
        
        # first step
        if len(rank) == 0:
            c_max = -1
            for j in nonrank:
                c_kj = np.linalg.norm(np.dot(T.T, X[:,j]), norm)
                if c_kj > c_max:
                    c_max = c_kj
                    j_max = j
            j_current = j_max  
            # save new input
            rank.append(j_current)
            nonrank.remove(j_current)
            # swap columns
            idx = len(rank)-1
            XA[:,idx] = X[:, j_current]
            y_min = 1
        
        # last step
        elif len(nonrank) == 0:  
            Y = Yk1
            W[rank] = Wk1
            y_min = 1

        # intermediate step        
        else:  
            Yk2 = (Yk1 - Y).T
            T2 = (T - Y).T
            
            X9 = X[:, j_current]
            c_max = np.linalg.norm(np.dot(T2, X9), norm)
            #c_max = np.linalg.norm(np.dot(T2, X[:, j_current]), norm)
            
            #fun = lambda y,x_new: (1-y)*c_max - np.linalg.norm(T2.dot(x_new) - y*Yk2.dot(x_new))
            fun_p = lambda y,p1,p2: (1-y)*c_max - np.linalg.norm(p1 - y*p2)  # super fast parametrized function
    
            # find optimal step (minimum over possible additional inputs) 
            y_min = 1  # upper interval
            for j_new in nonrank:
                x_new = X[:,j_new]
                # pre-calculate constant parts of the optimization function for the given x_new
                p1 = T2.dot(x_new)
                p2 = Yk2.dot(x_new)
                if (1-y_min)*c_max < np.linalg.norm(p1 - y_min*p2):  # skip optimization if min(fun) > y_min
                    try:
                        zero = 1E-15  # finding a value greater than zero
                        y_kj = optimize.brentq(fun_p, zero, y_min, xtol=1E-6, args=(p1,p2))
                        y_min = y_kj
                        j_min = j_new
                    except ValueError:  
                        # ValueError: f(a) and f(b) must have different signs
                        # here f(a) < 0 and f(b) < 0; does not fit our purposes anyway 
                        # ignoring this case
                        pass                  
                    
            if y_min == 1:  # if no suitable solution was found
                j_min = j_new
            j_current = j_min

            # add new input into model
            rank.append(j_current)
            nonrank.remove(j_current)
            # add new input to X matrix
            idx = len(rank)-1
            XA[:,idx] = X[:,j_current]

        # post-update ELM estimation with current set of inputs, with LU-ELM
        XtX = XX[rank,:][:,rank]
        XtT = XT[rank,:]
        LU, piv = lu_factor(XtX)#, overwrite_a=True)
        Wk1 = lu_solve((LU, piv), XtT)#, overwrite_b=True)
        X1 = XA[:,:len(rank)]  # replace fancy indexing with simple one
        Yk1 = np.dot(X1, Wk1)  

        if len(rank) > 1:
            # perform variable length step
            Y = (1-y_min)*Y + y_min*Yk1
            W = (1-y_min)*W     
            W[rank] += y_min*Wk1   
               
    # done, return ranking
    return rank

        


























































# -*- coding: utf-8 -*-
"""
Created on Sat Oct 18 17:21:12 2014

@author: akusok
"""

from setuptools import setup, Extension
#from distutils.core import setup

def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='hpelm',
      version='0.6.12',
      description='High-Performance implementation of an\
                   Extreme Learning Machine',
      long_description=readme(),
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: BSD License',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.7',
          'Topic :: Scientific/Engineering :: Information Analysis',
      ],
      keywords='ELM HPC regression classification ANN',
      url='https://www.researchgate.net/profile/Anton_Akusok',
      author='Anton Akusok',
      author_email='akusok.a@gmail.com',
      license='BSD (3-clause)',
      packages=['hpelm',
                'hpelm.modules',
                'hpelm.tests',
                'hpelm.acc'],
      install_requires=[
          'numpy',
          'numexpr',
          'scipy>=0.12',
          'tables',
          'cython'
      ],
      scripts=['bin/elm_naive.py'],
      test_suite='nose.collector',
      tests_require=['nose'],
      include_package_data=True,
      zip_safe=False)
